#include <Wire.h>
#include <PCA9557.h> // Include the I/O expander library
#include <RTClib.h> // Include the RTC library

#define GPIO_EXPANDER_ADDRESS 0x18
#define SDA_PIN 6
#define SCL_PIN 7

PCA9557 io(GPIO_EXPANDER_ADDRESS, &Wire);
RTC_DS3231 rtc; // Create an instance of the RTC object

void setup() {
  Serial.begin(9600);
  Wire.begin(SDA_PIN, SCL_PIN);
  
  if (!rtc.begin()) {
    Serial.println("Couldn't find RTC");
    while (1);
  }

  if (rtc.lostPower()) {
    // Set the RTC time to the compile time
    rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));
  }
 
  // Configure digital pins as inputs
  pinMode(4, INPUT);
  pinMode(5, INPUT);
  pinMode(10, INPUT);
  pinMode(11, INPUT);

  // Configure GPIO pins as outputs for relay control
  for (int i = 1; i <= 6; i++) {
    io.pinMode(i, OUTPUT);
  }
}

void loop() {
  DateTime now = rtc.now(); // Get current time from RTC

  // Read digital inputs and activate corresponding relays
  if (digitalRead(4) == HIGH) {
    io.digitalWrite(1, HIGH);
  } else {
    io.digitalWrite(1, LOW);
  }
  if (digitalRead(5) == HIGH) {
    io.digitalWrite(2, HIGH);
  } else {
    io.digitalWrite(2, LOW);
  }
  if (digitalRead(10) == HIGH) {
    io.digitalWrite(3, HIGH);
  } else {
    io.digitalWrite(3, LOW);
  }
  if (digitalRead(11) == HIGH) {
    io.digitalWrite(4, HIGH);
  } else {
    io.digitalWrite(4, LOW);
  }

  // Read analog inputs and activate relays 5 and 6 accordingly
  int analogValue1 = analogRead(0);
  int analogValue2 = analogRead(1);
  int analogValue3 = analogRead(2);
  int analogValue4 = analogRead(3);

  if (analogValue1 >= 1023 || analogValue2 >= 1023 || analogValue3 >= 1023 || analogValue4 >= 1023) {
    io.digitalWrite(5, HIGH); // Activate relay 5
  } else {
    io.digitalWrite(5, LOW); // Deactivate relay 5
  }

  if (analogValue1 >= 2047 || analogValue2 >= 2047 || analogValue3 >= 2047 || analogValue4 >= 2047) {
    io.digitalWrite(6, HIGH); // Activate relay 6
  } else {
    io.digitalWrite(6, LOW); // Deactivate relay 6
  }

  // Set PWM outputs based on analog input values
  analogWrite(18, calculatePWM(analogValue1)); // Analog Output 1
  analogWrite(19, calculatePWM(analogValue2)); // Analog Output 2
  analogWrite(20, calculatePWM(analogValue3)); // Analog Output 3
  analogWrite(21, calculatePWM(analogValue4)); // Analog Output 4

  // Prepare the timestamp
  char timestamp[20];
  sprintf(timestamp, "%04d-%02d-%02d %02d:%02d:%02d", now.year(), now.month(), now.day(), now.hour(), now.minute(), now.second());

  // Enhanced debug output with actual time/date stamp
  Serial.print(timestamp); Serial.print(" | ");
  Serial.print(digitalRead(4)); Serial.print("  ");
  Serial.print(digitalRead(5)); Serial.print("  ");
  Serial.print(digitalRead(10)); Serial.print("  ");
  Serial.print(digitalRead(11)); Serial.print("  ");
  Serial.print(io.digitalRead(1)); Serial.print("  ");
  Serial.print(io.digitalRead(2)); Serial.print("  ");
  Serial.print(io.digitalRead(3)); Serial.print("  ");
  Serial.print(io.digitalRead(4)); Serial.print("  ");
  Serial.print(io.digitalRead(5)); Serial.print("  ");
  Serial.print(io.digitalRead(6)); Serial.print("  ");
  Serial.print(analogRead(0)); Serial.print("  ");
  Serial.print(analogRead(1)); Serial.print("  ");
  Serial.print(analogRead(2)); Serial.print("  ");
  Serial.println(analogRead(3));

  delay(200); // Adjust delay as needed
}

// Function to calculate PWM duty cycle based on analog reading
int calculatePWM(int analogValue) {
  // Calculate PWM duty cycle using linear interpolation
  int pwm = map(analogValue, 0, 3300, 0, 255); // Adjusted range for analog input values

  // Ensure PWM duty cycle does not exceed 255
  return min(pwm, 255);
}
